function [ modelFunc_Y_convolved_binned ] = calculateDensityDistribution( sampleType, modelFuncType, modelFunctionSettings, samplingSettings, convolutionSettings )
% This function calculates the model function to be fit on the histogram
% data of the measurement. First, calls a function that calculates the
% function values of the supposed binding site density, then convolves it
% with the inaccuracies arising from the localization precision and the
% linker length, finally numerically integrates it over the histogram bins.

%% Calculate the convolution function

% get he simpling bin sizes for numerical integration:
modelFunc_sampling_dX=modelFunctionSampling.dX(samplingSettings.binningRegion, samplingSettings.sampling_N);

if ~convolutionSettings.iterGaussianConvBool
    % in this case the convolution function should already be calculated, prior to the
    % iteration
    
    convFunc=convolutionSettings.convFunc;
    
    % samplingSettings.convRad_N; % it should be ok
else
    % for calculating the cnvolution function within the iteration 
    
    % calculate the function with that the selected binding site density
    % function (model function) shall be convolved in order to get model
    % distribution to be fit on the data, accounts for the effect of the
    % linker radius, for the localisation precision and for an arbitrary
    % Gaussian smearing (the later one is fitted)
    GaussianConvSigma=modelFunctionSettings.GaussianConvSigma;
    [convFunc, convRad_N] = calculateConvolutionFunction(modelFunc_sampling_dX, convolutionSettings.linkerRad, convolutionSettings.locPrecision, convolutionSettings.linkerType, GaussianConvSigma);
    samplingSettings.convRad_N=convRad_N;
end

%% Model function prior the convolution

[ modelFunc_Y ] = calculateModelFunction( sampleType, modelFuncType, modelFunctionSettings, samplingSettings );

%% Convolution

% convolve the model function:
modelFunc_Y_convolved = convolveModelFunction(modelFunc_Y, convFunc, modelFunc_sampling_dX);

% convolve the background:
background=modelFunctionSettings.background;
if numel(background)==1
    background_convolved=background;
elseif numel(background)==numel(modelFunc_Y)
    background_convolved = convolveModelFunction(background, convFunc, modelFunc_sampling_dX);
else
    error('Invalid number of data points of the background.')
end

%% Set the model function height or its area ander the curve

if isfield(modelFunctionSettings, 'height') && isfield(modelFunctionSettings, 'area')
    error('Please give only the desired height or the  desired area of the model function, not both.')
end

% set the hight or the area of the convolved model function:
if isfield(modelFunctionSettings, 'height')
    modelFunc_Y_convolved=modelFunctions.setAHeight(modelFunc_Y_convolved, modelFunctionSettings.height);
elseif isfield(modelFunctionSettings, 'height1')
    modelFunc_Y_convolved=modelFunctions.setAHeight(modelFunc_Y_convolved, max(modelFunctionSettings.height1,modelFunctionSettings.height2));
elseif isfield(modelFunctionSettings, 'area')
    modelFunc_Y_convolved=modelFunctions.setArea(modelFunc_Y_convolved, modelFunc_sampling_dX, modelFunctionSettings.area, background_convolved);
elseif isfield(modelFunctionSettings, 'gapDepth')
    modelFunc_Y_convolved=modelFunctions.setAHeight(modelFunc_Y_convolved, modelFunctionSettings.background+modelFunctionSettings.gapDepth + modelFunctionSettings.rimHeight);
else
    error('Either height or area value has to be given for the model function.')
end

%% Add the background

modelFunc_Y_convolved=modelFunc_Y_convolved+background_convolved;

%% Numerical integration of the convolved model function, for comparison with the measured histogram data

modelFunc_Y_convolved_binned = binModelFunction(modelFunc_Y_convolved, samplingSettings.histBinN, samplingSettings.binRefinement);


end

