classdef getInitialParameters
% These methods automatically determines (hopefully meaningful) initial
% parameters for the different kind of samples and model functions based
% on the histogram of the original data.
    
    methods (Static)
        
        function [initialParameters, constrains] = singleLine(histogramStruct, modelFuncType, modelFunctionSettings, fittingParameterList)
            % the histogram data:
            histCounts=histogramStruct.histCounts;
            histEdges=histogramStruct.histEdges;
            histN=numel(histCounts);
            histX=(histEdges(1:end-1)+histEdges(2:end))/2;
            
            % initialze the initial parameters and the constrains vector
            initialParameters=zeros(numel(fittingParameterList), 1);
            constrains=zeros(numel(fittingParameterList), 2);
            
            % set the backround before the other parameters
            backgroundBoolVect=strcmp(fittingParameterList, 'background');
            if sum(strcmp(fittingParameterList, 'background'))
                % determine the background for the few marginal bins
                histN_forBG=floor(histN/8);
                % parameter for constant background
                initialParameters(backgroundBoolVect)=mean([mean(histCounts(1:histN_forBG)), mean(histCounts(end-histN_forBG:end))]);
                constrains(backgroundBoolVect,:)=[0, Inf];
                
                background=initialParameters(backgroundBoolVect);
            else
                if isfield(modelFunctionSettings, 'background')
                    background=modelFunctionSettings.background;
                else
                    error('The background for the model function (single line) has to be set or to be fitted.')
                end
            end
            
            % set the central position before the other parameters
            positionBoolVect=strcmp(fittingParameterList, 'position');
            if sum(strcmp(fittingParameterList, 'position'))
                % central position of the single line structure:
                initialParameters(positionBoolVect)=sum((histCounts-background).*histX/sum(histCounts-background));
                constrains(positionBoolVect,:)=[-Inf, Inf];
                
                position=initialParameters(positionBoolVect);
            else
                if isfield(modelFunctionSettings, 'position')
                    position=modelFunctionSettings.position;
                else
                    error('The position for the model function (single line) has to be set or to be fitted.')
                end
            end
            
            for idxParam=1:numel(fittingParameterList)
                switch fittingParameterList{idxParam}
                    case 'width'
                        % parameter determining its width, calculate it from the standard deviation of the data:
                        initialParameters(idxParam)=2.0*sqrt(sum(((histCounts-background).*(histX-position).^2))/((sum(histCounts-background)-1)));
                        constrains(idxParam,:)=[0, Inf];
                    case 'area'
                        % the area under the histogram:
                        initialParameters(idxParam)=sum(histCounts)*hist_dX;
                        constrains(idxParam,:)=[0, Inf];
                    case 'height'
                        % parameter determining its peak value:
                        initialParameters(idxParam)=max(histCounts);
                        constrains(idxParam,:)=[0, Inf];
                    case 'position'
                        % do nothing
                    case 'background'
                        % do nothing
                    case 'GaussianConvSigma'
                        % additional Gaussian smearing
                        % let it be:
                        initialParameters(idxParam)=abs(histEdges(2)-histEdges(1))*0.01;
                        constrains(idxParam,:)=[0, Inf];
                    otherwise
                        error('Invalid parameter was given to be fit (single line fitting).')
                end
            end
        end
        
        
        function [initialParameters, constrains] = doubleLine(histogramStruct, modelFuncType, modelFunctionSettings, fittingParameterList)
            
            % the histogram data:
            histCounts=histogramStruct.histCounts;
            histEdges=histogramStruct.histEdges;
            histN=numel(histCounts);
            histX=(histEdges(1:end-1)+histEdges(2:end))/2;
            
            % initialze the initial parameters and the constrains vector
            initialParameters=zeros(numel(fittingParameterList), 1);
            constrains=zeros(numel(fittingParameterList), 2);
            
            % determine the initial double peak parameters with double
            % Gaussian fittion:
            if exist('OCTAVE_VERSION', 'builtin') ~= 0
                % when run in Octave, detemine the initial parameters with and
                % external peak fitting tool
                % source: https://terpconnect.umd.edu/~toh/spectrum/InteractivePeakFitter.htm#command
                
                % fit two unconstrained Gaussians with linear baseline (could not find constant baseline setting...),
                % without plotting and with the default settings:
                [FitResults,FitError,baseline] = peakfit([hist_X, histCounts],0,0,2,0,0,0,0,1,0,0);
                if FitError(1)>5
                    % it the fit error is larger than 5%, try it without
                    % background removal
                    % I think the linear background removal does not work the
                    % way I think it should work...
                    % and the "peakfit" function does not run the same in
                    % Octave and in Matlab, it uses "fminseach" which I guess
                    % runs a slightly different step sizes
                    [FitResults2,FitError2,baseline2] = peakfit([hist_X, histCounts],0,0,2,0,0,0,0,0,0,0);
                    if FitError2(1)<FitError(1)
                        FitResults=FitResults2;
                        FitError=FitError2;
                        baseline=baseline2;
                    end
                end
                
                % flat baseline this should be used, but does not work:
                % [FitResults,~,baseline] = peakfit([hist_X, histCounts],0,0,2,0,0,0,0,3,0,0);
                
                % The first dimension of the "FittedResults" corresponds to the
                % fitted peak, the second dimension correspond to the fitted
                % parameters of the peaks
                % The order of the paramerers (along the second dimension):
                % FittedResults=['peak index', 'position', 'height', 'width', 'area']
                
                % the constant to convert the FWHM to the "standard deviation"
                % 2*sqrt(2*log(2))
                widthConst=1/2.3548;
                
                densFuncMax_init_1=FitResults(1,3);   % parameter determining its peak value
                densFuncMax_init_2=FitResults(2,3);   % parameter determining its peak value
                densFuncBG_init=baseline;          % parameter for constant background
                densFuncSize_init_1=FitResults(1,4)*widthConst;
                densFuncSize_init_2=FitResults(2,4)*widthConst;       % parameter determining its width
                pos1_init=FitResults(1,2);       % initial position of the first line
                pos2_init=FitResults(2,2);       % initial position of the second line
            else
                % detemine the initial parameters with Matlab's built-in double
                % Gaussian fitting function
                
                f=fit(histX, histCounts, 'gauss2');
                C=coeffvalues(f);
                
                % the constant to convert the FWHM to the "standard deviation"
                % 2*sqrt(2*log(2))
                widthConst=1/2.3548;
                
                % initial parameters of the selected dye density function for caclculating the model function
                %densFuncMax_init=(C(1,1)+C(1,4))/2;   % parameter determining its peak value
                densFuncMax_init_1=C(1,1);   % parameter determining its peak value
                densFuncMax_init_2=C(1,4);   % parameter determining its peak value
                %densFuncBG_init=0;          % parameter for constant background
                densFuncSize_init_1=C(1,3)/sqrt(2);       % parameter determining its width, the standard deviation
                densFuncSize_init_2=C(1,6)/sqrt(2);       % parameter determining its width, the standard deviation
                pos1_init=C(1,2);       % initial position of the first line
                pos2_init=C(1,5);       % initial position of the second line
                
                
                % determine the background for the few marginal bins
                histN_forBG=floor(histN/8);
                % parameter for constant background
                densFuncBG_init=max(max(histCounts(1:histN_forBG)), max(histCounts(end-histN_forBG:end)));       % parameter determining its width
            end
            
            for idxParam=1:numel(fittingParameterList)
                switch fittingParameterList{idxParam}
                    case 'width'
                        % let it be the avera of the two peak widths:
                        initialParameters(idxParam)=(densFuncSize_init_1+densFuncSize_init_2)/2;
                        constrains(idxParam,:)=[0, Inf];
                    case 'area'
                        % the area under the histogram:
                        initialParameters(idxParam)=sum(histCounts)*hist_dX;
                        constrains(idxParam,:)=[0, Inf];
                    case 'height'
                        % parameter determining its peak value:
                        initialParameters(idxParam)=(densFuncMax_init_1+densFuncMax_init_2)/2;
                        constrains(idxParam,:)=[0, Inf];
                    case 'height1'
                        % height of the first peak:
                        initialParameters(idxParam)=densFuncMax_init_1;
                        constrains(idxParam,:)=[0, Inf];
                    case 'height2'
                        % height of the second peak:
                        initialParameters(idxParam)=densFuncMax_init_2;
                        constrains(idxParam,:)=[0, Inf];
                    case 'position1'
                        %position of the first peak:
                        initialParameters(idxParam)=pos1_init;
                        constrains(idxParam,:)=[-Inf, Inf];
                    case 'position2'
                        %position of the second peak:
                        initialParameters(idxParam)=pos2_init;
                        constrains(idxParam,:)=[-Inf, Inf];
                    case 'background'
                        % the background on the peaks:
                        initialParameters(idxParam)=densFuncBG_init;
                        constrains(idxParam,:)=[0, Inf];
                    case 'GaussianConvSigma'
                        % additional Gaussian smearing
                        % let it be:
                        initialParameters(idxParam)=abs(histEdges(2)-histEdges(1))*0.01;
                        constrains(idxParam,:)=[0, Inf];                    otherwise
                        error('Invalid parameter was given to be fit (single line fitting).')
                end
            end
            
        end
        
        
        function [initialParameters, constrains] = gap(histogramStruct, modelFuncType, modelFunctionSettings, fittingParameterList, initFitVals)
            
            % the histogram data:
            histCounts=histogramStruct.histCounts;
            histEdges=histogramStruct.histEdges;
            histN=numel(histCounts);
            histX=(histEdges(1:end-1)+histEdges(2:end))/2;
            
            % initialze the initial parameters and the constrains vector
            initialParameters=zeros(numel(fittingParameterList), 1);
            constrains=zeros(numel(fittingParameterList), 2);
            
            % set the backround before the other parameters
            backgroundBoolVect=strcmp(fittingParameterList, 'background');
            if sum(strcmp(fittingParameterList, 'background'))
                % let the initial background be the histogram's central
                % bin's value
                centerIdx=floor(length(histCounts)/2);
                initialParameters(backgroundBoolVect)=histCounts(centerIdx);
                constrains(backgroundBoolVect,:)=[0, Inf];
                
                background=initialParameters(backgroundBoolVect);
            else
                if isfield(modelFunctionSettings, 'background')
                    background=modelFunctionSettings.background;
                else
                    error('The background for the model function (gap) has to be set or to be fitted.')
                end
            end
        
            
            % set the rim width before the other parameters
            rimWidthBoolVect=strcmp(fittingParameterList, 'rimWidth');
            if sum(strcmp(fittingParameterList, 'rimWidth'))
                % determining the initial gap depht
                initialParameters(rimWidthBoolVect)=(histCounts(1)+histCounts(end))/2-background;
                constrains(rimWidthBoolVect,:)=[0, Inf];
                
                rimWidth=initialParameters(rimWidthBoolVect);
            else
                if isfield(modelFunctionSettings, 'rimWidth')
                    rimWidth=modelFunctionSettings.rimWidth;
                else
                    error('The rim width for the model function (gap) has to be set or to be fitted.')
                end
            end
            
            % set the gap depht before the other parameters
            gapDepthBoolVect=strcmp(fittingParameterList, 'gapDepth');
            if sum(strcmp(fittingParameterList, 'gapDepth'))
                % determining the initial gap depht
                histN_forGapDepth=floor(histN/8);
                % parameter for constant background
                initialParameters(gapDepthBoolVect)=mean([mean(histCounts(1:histN_forGapDepth)), mean(histCounts(end-histN_forGapDepth:end))])-background;
                constrains(gapDepthBoolVect,:)=[0, Inf];
                
                gapDepth=initialParameters(gapDepthBoolVect);
            else
                if isfield(modelFunctionSettings, 'gapDepth')
                    gapDepth=modelFunctionSettings.gapDepth;
                else
                    error('The gap width for the model function (gap) has to be set or to be fitted.')
                end
            end
            
            for idxParam=1:numel(fittingParameterList)
                switch fittingParameterList{idxParam}
                    case 'gapWidth'
                        % parameter determining its width
                        initialParameters(idxParam)=initFitVals.gap.densFuncSize_init;
                        constrains(idxParam,:)=[0, Inf];
                        
                        % TODO: megfontolni, hogy a szélességet ne egy
                        % külső paraméterből vegye, hanem az invertált
                        % hisztogramból
                    case 'rimWidth'
                        % the widths of the rim of the gap
                        initialParameters(idxParam)=initFitVals.gap.rimWidth;
                        constrains(idxParam,:)=[0, Inf];
                        
                        rimWidth=initialParameters(idxParam);
                    case 'area'
                        % the area under the histogram:
                        initialParameters(idxParam)=sum(histCounts)*hist_dX;
                        constrains(idxParam,:)=[0, Inf];
                    case 'gapDepth'
                        % do nothing
                    case 'position'
                        invertedHist=gapDepth+background-histCounts;
                        invertedHist(invertedHist<0)=0;
                        initialParameters(idxParam)=sum(invertedHist.*histX/sum(invertedHist));
                        constrains(idxParam,:)=[-Inf, Inf];
                    case 'rimHeight'
                        % do nothing
                    case 'background'
                        % do nothing
                    case 'GaussianConvSigma'
                        % additional Gaussian smearing
                        % let it be:
                        initialParameters(idxParam)=abs(histEdges(2)-histEdges(1))*0.01;
                        constrains(idxParam,:)=[0, Inf];
                    otherwise
                        error('Invalid parameter was given to be fit (single line fitting).')
                end
            end        
            
            % set the rim height after the other parameters
            rimHeightBoolVect=strcmp(fittingParameterList, 'rimHeight');
            if sum(strcmp(fittingParameterList, 'rimHeight'))
                oldwayBool=false;
                if oldwayBool % old way
                % determining the initial rim height
                rimAreaConst=50;
                % I don't know why I calculated this way back then:
                initialParameters(rimHeightBoolVect)=gapDepth*rimAreaConst/rimWidth;
                else
                    initialParameters(rimHeightBoolVect)=max(histCounts)-background-gapDepth;
                end
                constrains(rimHeightBoolVect,:)=[0, Inf];
                
                rimHeight=initialParameters(rimHeightBoolVect);
            else
                if isfield(modelFunctionSettings, 'rimHeight')
                    rimHeight=modelFunctionSettings.rimHeight;
                else
                    error('The rim height for the model function (gap) has to be set or to be fitted.')
                end
            end
            
        end
    end
    
end

