function [fittedParameters, FVAL] = lineDiameterFitting(histogramStruct, modelFunctionStruct, algorithm, fittingParameterList, initialParameters, constrains, options)
% Fitting the model function to the histogram data of the measured
% distances.

% The structure of the input structures:
% modelFunctionStruct
%     sampleType
%     modelFuncType
%     modelFunctionSettings
%         width
%         height
%         background
%         position
%         ...
%     samplingSettings
%         binningRegion
%         sampling_N
%         binRefinement
%         convRad_N
%         histBinN
%     convolutionSettings
%         convFunc
%         locPrecision
%         iterGaussianConvBool
%         linkerType
% histogramStruct
%     histCounts
%     histEdges

% options parameter structure for the fitting algrithm:
if isempty(options)
    options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
    %options = optimset();
else
    % do nothing
    % the user has already set them (properly, hopefully)
end


switch algorithm
    case "fmincon"
        % constrains for the fitting:
            
        constrains_lower=zeros(size(constrains, 1), 1);
        constrains_lowerupper=zeros(size(constrains, 1), 1);
        
        [fittedParamsVect,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) residuum(x, histogramStruct, modelFunctionStruct, fittingParameterList), initialParameters, [], [], [], [], constrains_lower, constrains_lowerupper, [], options);
    case "fminsearch"
        [fittedParamsVect,FVAL,EXITFLAG,OUTPUT]=fminsearch(@(x) residuum(x, histogramStruct, modelFunctionStruct, fittingParameterList), initialParameters, options);
    case "fminunc"
        [fittedParamsVect,FVAL,EXITFLAG,OUTPUT]=fminunc(@(x) residuum(x, histogramStruct, modelFunctionStruct, fittingParameterList), initialParameters, options);
end

% update the model function parameters with the fitted ones:
fittedParameters=modelFunctionStruct.modelFunctionSettings;
for idxParam=1:numel(fittedParamsVect)
    fittedParameters.(fittingParameterList{idxParam})=fittedParamsVect(idxParam);
end

end


function res=residuum(x, histogramStruct, modelFunctionStruct, fittingParameterList)
% This function calculates the residuum that is used for
% fitting the model function to the histogram original data of
% the measured central line - localiztion distance distribution.

% passing the settings of the model function:
sampleType=modelFunctionStruct.sampleType;
modelFuncType=modelFunctionStruct.modelFuncType;
modelFunctionSettings=modelFunctionStruct.modelFunctionSettings;
samplingSettings=modelFunctionStruct.samplingSettings;
convolutionSettings=modelFunctionStruct.convolutionSettings;

% parameters to fit:
for idxParameter=1:numel(x)
    modelFunctionSettings.(fittingParameterList{idxParameter})=x(idxParameter);
end

% values of the model function:
[ modelFunc_Y_convolved_binned ] = calculateDensityDistribution(sampleType, modelFuncType, modelFunctionSettings, samplingSettings, convolutionSettings );

% passing the histogram counts of the original data:
histCounts=histogramStruct.histCounts;

% residual of the fitted model function:
res=sum((modelFunc_Y_convolved_binned-histCounts).^2)/sum(histCounts.^2);      % mean squared error as the residue

%% FOR TESTING:
% histEdges=histogramStruct.histEdges;
% histX=(histEdges(1:end-1)+histEdges(2:end))/2;
% figure(55)
% histogram('BinCounts', histCounts', 'BinEdges', histEdges')
% hold on;
% plot(histX, modelFunc_Y_convolved_binned)
% hold off;
% res
end
