function [ fittedParamsTable, fittedHistFigure ] = lineDiameter_fit( lineDiam_histData, curveFitTool, ...
    linkerType, sampleType, varargin )
% TDOD: Megfontolni, hogy a a fggvnyen kvl legyen helyezve a a kezdeti
% paramterek, s az egyb belltosok megdasa ('modelFunctionParameters',
% 'fittingParameterList', 'initialParameters' s 'constrains' vltozk)
% Ekkor taln egyesteni is lehetne a kdot a
% 'lineDiameter_singlerectfit'-tel.
% Ekkor a 'rainSTORM_extras_lineDiameter_fit' szkript-en is vltoztatni
% kne valamennyit.


% This function reads the histogram data of the measured distances and
% fits a model function to it. Can be used for determining a line diameter.
% At first, a dialogue windows is asking the fitting parameters.
% Then function prepares the 'central curve-localization' data. If
% no localization precision filtering is applied and the histogram edges
% match for all lines, then the histogram data is used, otherwise, new
% histogram is created from the raw "central curve-localization" data. The
% sampling points for the convolution and for the model function are also
% calculated.
% In the next step the model function fitting is performed. During the fit,
% the iteration changes the binding site distribution function, then its is
% convolved with the localization inaccuracies and compared with the raw
% data of the 'central curve-localization' distribution. In case of a 
% single line sample the initial width is derived from the standard
% deviation of the histogram data while in case of a double line sample, it
% is determined by a double Guassian fit. In case a single line sample the
% expected model function widths are also calculated from the standard deviation.
% First the standard deviation of the fitted constant background is
% subtracted from the standard deviation of the data, then the the width
% is calculated for the 'Gaussian', 'Rectangular' and 'Disk' model
% functions. Finally a figure is created showing the histogram data, the
% fitted model function and the fitted binding site density function. The
% fitted model function widths ("FWHM"), the residual of the fit and in case
% of double line sample, the distance of the lines are also written out on
% the figure.

% parents: rainSTORM_extras_lineDiameter_fit

% INPUT VARIABLES
% lineDiam_histData: raw data of the localization-central line distances, its localization precision and of the histogram created from it
% linkerType: type of distribution associated with the linker length, can be 'sphere' or 'Gaussian'
% sampleType: type of the sample, single line, double line or gap
% varargin: binding site density parameters 

%% Other input parameters
if length(varargin)==1
    
    dialogOutput=lineDiameter_fit_inputDialog( ThompsonPrec );
    
    modelFuncType=dialogOutput.modelFuncType;
    linkerRad=dialogOutput.linkerLength;
    precBounds=dialogOutput.precBounds;
    locPrecision_manuallySet=dialogOutput.singlePrecValue;
    multiPrecBool=dialogOutput.multiPrecBool;
    initFitVals=varargin{1};
    % TODO revise for the input dialog
    iterGaussianConvBool=false;
elseif length(varargin)==7
    modelFuncType=varargin{1};
    linkerRad=varargin{2};
    precBounds=varargin{3};
    locPrecision_manuallySet=varargin{4};
    multiPrecBool=varargin{5};
    initFitVals=varargin{6};
    iterGaussianConvBool=varargin{7};
else
    error('Number of the function inputs is not correct.')
end

%% Reading the measured data of line diameter

% double array containing the distance data and the precision of the
% localizations
locLineDist=lineDiam_histData.locLineDist;
ThompsonPrec=lineDiam_histData.ThompsonPrec;

% cell array containing the histogram edges of the different lines:
histEdges_cell=lineDiam_histData.histEdges;
% cell array containing the histogram counts of the different lines:
histCounts_cell=lineDiam_histData.histCounts;

%% Preparing the measured data of the line

[histCounts, histEdges] = lineDiameter_fit_getHistogram(histEdges_cell, histCounts_cell, locLineDist, ThompsonPrec, precBounds);

histogramStruct.histCounts=histCounts;
histogramStruct.histEdges=histEdges;

locPrecision = lineDiameter_fit_getPrecision(locPrecision_manuallySet, ThompsonPrec, precBounds, multiPrecBool);

%% The model function sampling settings

% refinement for the model function sampling:
binRefinement=10;
% Every bin (of the measured histogram data) should be refined to this many
% points. This determines only the sampling pont number, the sampling
% points will be equidistants, no matter the original bin sizes (thats why
% they should be equally sized).

[histBinN, binningRegion, sampling_N] = modelFunctionSampling.settings(histEdges ,binRefinement);

samplingSettings.binningRegion=binningRegion;
samplingSettings.sampling_N=sampling_N;
samplingSettings.binRefinement=binRefinement;
samplingSettings.convRad_N=[];
samplingSettings.histBinN=histBinN;

%% The model function settings
% Only those model function paramerters has to be set that the fitting is
% not performed on.
% The "modelFunctionSettings" should be in line with the
% "fittingParameterList" variable. What is defined in the
% "modelFunctionSettings" that is not respected in the
% "modelFunctionSettings" and will be overwrited (the fitting will be
% performed on that and will not be held constant). Every parameter
% neccessary parameter that is not defined in "fittingParameterList" (on
% which the fitting is not performed), should be defined in
% "modelFunctionSettings".

% initialize the structure for the model function parameters:
modelFunctionParameters=struct();

% the additional Gaussian smearing:
modelFunctionParameters.GaussianConvSigma=0;

% ring thickness in case of the "ring" structure
modelFunctionParameters.ringThickness=250;

% for predefined background:
%modelFunctionParameters.background=0;

% for predefined function area:
%modelFunctionParameters.area=sum(histCounts);
% if it is set, the fitting should not be performed on the "height"

% for rimless gap structure:
%modelFunctionParameters.rimWidth=0;
%modelFunctionParameters.rimHeight=0;

%% The convolution (if can be performed prior the fitting)

if ~iterGaussianConvBool
    % if the iteration should also run on the width of the Gaussian convolution

    % calculate the function with that the selected binding site density
    % function (model function) shall be convolved in order to get model
    % distribution to be fit on the data, accounts for the effect of the
    % linker radius and for the localisation precision
    modelFunc_sampling_dX=modelFunctionSampling.dX(binningRegion, sampling_N);
    [convFunc, convRad_N]=calculateConvolutionFunction(modelFunc_sampling_dX, linkerRad, locPrecision, linkerType, modelFunctionParameters.GaussianConvSigma);
    
    % TESTING:
    % convFunc=ones(31,1);
    % convRad_N=15;
else
    % calculate the convolution function inside the function called by the
    % iteration
    convFunc=[];
    convRad_N=[];
end


convolutionSettings.convFunc=convFunc;
convolutionSettings.locPrecision=locPrecision;
convolutionSettings.iterGaussianConvBool=iterGaussianConvBool;
convolutionSettings.linkerType=linkerType;
convolutionSettings.linkerRad=linkerRad;

%% Settings for the model function
samplingSettings.convRad_N=convRad_N;

modelFunctionStruct.sampleType=sampleType;
modelFunctionStruct.modelFuncType=modelFuncType;
modelFunctionStruct.modelFunctionSettings=modelFunctionParameters;
modelFunctionStruct.samplingSettings=samplingSettings;
modelFunctionStruct.convolutionSettings=convolutionSettings;

%% Fitting the model function
switch sampleType
    case 'single line'

        algorithm='fminsearch';
        %algorithm='fmincon';
        %algorithm='fminunc';
        
        fittingParameterList={'width', 'height', 'position', 'background'};
        
        if ~iterGaussianConvBool
            % do nothing
        else
            fittingParameterList=[fittingParameterList, 'GaussianConvSigma'];
        end
        
        [initialParameters, constrains]=getInitialParameters.singleLine(histogramStruct, modelFuncType, modelFunctionParameters, fittingParameterList);
        
        % fitting the model function to the histogram data of the measured
        % distances
        options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
        %options = optimset();
        
        [fittedParameters, FVAL]= lineDiameterFitting(histogramStruct, modelFunctionStruct, algorithm, fittingParameterList, initialParameters, constrains, options);
        
        % the fitted parameters of the selected dye density function
        densFuncWidth_fitted=fittedParameters.width;        % parameter determining its width, the FWHM
        densFuncHeight_fitted=fittedParameters.height;         % parameter determining its peak value
        densFuncBG_fitted=fittedParameters.background;          % parameter for constant background
        
        if iterGaussianConvBool
            GaussianConvSigma_fitted=fittedParameters.GaussianConvSigma;
        else
            GaussianConvSigma_fitted=modelFunctionParameters.GaussianConvSigma;
        end
        
        % passing the fitted parameters into a table
        fittedParamsTable=tableReplacement.data(densFuncWidth_fitted, densFuncHeight_fitted, densFuncBG_fitted);
        
    case 'double lines'

        algorithm='fminsearch';
        %algorithm='fmincon';
        %algorithm='fminunc';
        
        fittingParameterList={'width', 'height1', 'height2', 'position1', 'position2', 'background'};
        
        if ~iterGaussianConvBool
            % do nothing
        else
            fittingParameterList=[fittingParameterList, 'GaussianConvSigma'];
        end
        
        [initialParameters, constrains]=getInitialParameters.doubleLine(histogramStruct, modelFuncType, modelFunctionParameters, fittingParameterList);
        
        % fitting the model function to the histogram data of the measured
        % distances
        options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
        
        [fittedParameters, FVAL]= lineDiameterFitting(histogramStruct, modelFunctionStruct, algorithm, fittingParameterList, initialParameters, constrains, options);
        
        densFuncWidth_fitted=fittedParameters.width;        % parameter determining its width, the FWHM
        densFuncMax_fitted_1=fittedParameters.height1;         % parameter determining its peak value
        densFuncMax_fitted_2=fittedParameters.height2;         % parameter determining its peak value
        densFuncBG_fitted=fittedParameters.background;          % parameter for constant background
        pos_1_fitted=fittedParameters.position1;
        pos_2_fitted=fittedParameters.position2;
        lineDistance_fitted=abs(pos_1_fitted-pos_2_fitted);
        
        if iterGaussianConvBool
            GaussianConvSigma_fitted=fittedParameters.GaussianConvSigma;
        else
            GaussianConvSigma_fitted=modelFunctionParameters.GaussianConvSigma;
        end
        
        % passing the fitted parameters into a table
         fittedParamsTable=tableReplacement.data(densFuncWidth_fitted, densFuncMax_fitted_1, densFuncMax_fitted_2, densFuncBG_fitted, pos_1_fitted, pos_2_fitted, lineDistance_fitted);
        
    case 'gap'
        
        algorithm='fminsearch';
        %algorithm='fmincon';
        %algorithm='fminunc';
        
        fittingParameterList={'gapWidth', 'gapDepth', 'rimWidth', 'rimHeight', 'position', 'background'};
        
        if ~iterGaussianConvBool
            % do nothing
        else
            fittingParameterList=[fittingParameterList, 'GaussianConvSigma'];
        end
        
        [initialParameters, constrains]=getInitialParameters.gap(histogramStruct, modelFuncType, modelFunctionParameters, fittingParameterList, initFitVals);
        
        options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
        
        [fittedParameters, FVAL]= lineDiameterFitting(histogramStruct, modelFunctionStruct, algorithm, fittingParameterList, initialParameters, constrains, options);
        
        
        % the fitted parameters of the selected dye density function
        densFuncWidth_fitted=fittedParameters.gapWidth;        % parameter determining its width
        densFuncHeight_fitted=fittedParameters.gapDepth;         % parameter determining its peak value
        densFuncBG_fitted=fittedParameters.background;          % parameter for constant background
        rimHeight_fitted=fittedParameters.rimHeight;
        rimWidth_fitted=fittedParameters.rimWidth;
        % TODO: check it: Checked
        % warning('this one should be checked, is it correct?')
        rimDist_fitted=(densFuncWidth_fitted + rimWidth_fitted)/2;
        
        if iterGaussianConvBool
            GaussianConvSigma_fitted=fittedParameters.GaussianConvSigma;
        else
            GaussianConvSigma_fitted=modelFunctionParameters.GaussianConvSigma;
        end
        
        % passing the fitted parameters into a table
        if iterGaussianConvBool
            fittedParamsTable=tableReplacement.data(densFuncWidth_fitted, densFuncHeight_fitted, densFuncBG_fitted, rimHeight_fitted, rimDist_fitted, GaussianConvSigma_fitted);
        else
            fittedParamsTable=tableReplacement.data(densFuncWidth_fitted, densFuncHeight_fitted, densFuncBG_fitted, rimHeight_fitted, rimDist_fitted);
        end
end

% update the model function parameters with the fitted ones
fittedParameterNames=fieldnames(fittedParameters);
for idxFittedParam=1:numel(fittedParameterNames)
    modelFunctionParameters.(fittedParameterNames{idxFittedParam})=fittedParameters.(fittedParameterNames{idxFittedParam});
end
modelFunctionStruct.modelFunctionSettings=modelFunctionParameters;


%% Visualizing the result

% update convolution function
if ~iterGaussianConvBool
    % do nothing
else
    modelFunc_sampling_dX=modelFunctionSampling.dX(binningRegion, sampling_N);
    [convFunc, convRad_N] = calculateConvolutionFunction(modelFunc_sampling_dX, linkerRad, locPrecision, linkerType, GaussianConvSigma_fitted);
    
    % update the convolution settigns structure
    convolutionSettings.convFunc=convFunc;
    convolutionSettings.convRad_N=convRad_N;
    modelFunctionStruct.convolutionSettings=convolutionSettings;
end

switch sampleType
    case 'single line'
        fittedHistFigure = lineDiameter_fit_visualization.singleLine(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, densFuncBG_fitted, FVAL);
    case 'double lines'
        fittedHistFigure = lineDiameter_fit_visualization.doubleLine(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, lineDistance_fitted, FVAL);
    case 'gap'
        fittedHistFigure = lineDiameter_fit_visualization.gap(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, GaussianConvSigma_fitted, FVAL);
end

% show figure
% figure(fittedHistFisgure)

end
