function [ output ] = lineDiameter_fit_inputDialog( ThompsonPrec, sampleType, modelFuncType, linkerRad, locPrecision, multiPrecBool )
% This function generates the GUI elements for the line diameter fitting
% dialog, asks the parameters for the model function fitting on the line
% diameter data and also creates a histogram in the window showing the
% localization precision distribution (that is contained in the
% "ThompsonPrec" input variable). By clicking on the "fit" button, it
% gives back the set parameters and deletes the window.

% parents: lineDiameter_fit
% children: -

% TODO: revise!!!!! (the lineDiameter_fit has been modified on 2020.07.06.)

% the dialog, parent of all other elements
d = dialog('Position',[300 300 750 300],'Name','Select One');

% field for choosing the type of dye distribution
uicontrol('Parent',d,...
    'Style','text',...
    'HorizontalAlignment','center',...
    'Position',[90 250 210 40],...
    'String','Select the sample type:');

switch sampleType
    case {'single line', 'double lines'}
        modelFuncType_field=uicontrol('Parent',d,...
            'Style','popup',...
            'Position',[150 240 100 25],...
            'String',{'Gaussian'; 'Lorentzian'; 'Disk'; 'Circle'; 'Rectangular'}...
            );
        modelFuncType_list=get(modelFuncType_field, 'String');
        modelFuncType_index=find(strcmp(modelFuncType_list, modelFuncType));
        set(modelFuncType_field, 'Value', modelFuncType_index);
    case 'gap'
        modelFuncType_field=uicontrol('Parent',d,...
            'Style','popup',...
            'Position',[150 240 100 25],...
            'String',{'Gap'}...
            );
end

% field for giving the linker length
uicontrol('Parent',d,...
    'Style','text',...
    'HorizontalAlignment','left',...
    'Position',[50 195 210 25],...
    'String','Linker Length [nm]:');

linker_length_field=uicontrol('Parent',d,...
    'style','edit',...
    'String',num2str(linkerRad),...
    'Position',[215 200 100 25]);

% field for giving the precision bounds
uicontrol('Parent',d,...
    'Style','text',...
    'HorizontalAlignment','left',...
    'Position',[50 165 210 25],...
    'String','Precision Bounds [nm]:');

prec_bounds_field=uicontrol('Parent',d,...
    'style','edit',...
    'String','-Inf Inf',...
    'Position',[215 170 100 25]);


% button group for the raido buttons
bg = uibuttongroup('Parent',d,...
    'Visible','off',...
    'Unit', 'Pixel',...
    'Position',[65 100 235 55],...
    'SelectionChangedFcn',@(hObject, eventdata)bselection(hObject, eventdata));

%'Position',[0.1 0.5 0.6 0.2],...

% Create three radio buttons in the button group.
uicontrol(bg,'Style','radiobutton',...
    'String','Use  distinct precision',...
    'Tag', 'single_prec',...
    'Position',[10 30 225 20],...
    'HandleVisibility','off');

multi_prec_radiobutton=uicontrol(bg,'Style','radiobutton',...
    'String','Integrate over the precisions',...
    'Tag', 'multi_prec',...
    'Position',[10 5 225 20],...
    'Value', multiPrecBool,...
    'HandleVisibility','off');

% Make the uibuttongroup visible after creating child objects.
set(bg, 'Visible', 'on');

% input field if a single precision value should be used for the fitting
uicontrol('Parent',d,...
    'Style','text',...
    'HorizontalAlignment','left',...
    'Position',[50 55 210 25],...
    'String','Precision Value:');

single_prec_field=uicontrol('Parent',d,...
    'style','edit',...
    'String',num2str(locPrecision),...
    'Position',[215 60 100 25],...
    'Tag', 'sinlge_rec_field');

if ~multiPrecBool
    set(single_prec_field, 'Enable', 'On')
else
    set(single_prec_field, 'Enable', 'Off')
end


% button to initiating fitting
fitButton = uicontrol('Parent',d,...
    'Position',[200 20 70 25],...
    'String','Fit',...
    'Callback',@(hObject, eventdata)fitButton_callback(hObject, eventdata));

% button for closing the dialog
cancelButton = uicontrol('Parent',d,...
    'Position',[100 20 70 25],...
    'String','Cancel',...
    'Callback','delete(gcf)');

% add the graph containing the localization precision histogram

graph = axes('Parent', d,...
    'Units', 'normalized',...
    'Position', [0.53 0.2 0.45 0.7]);
    
    binEdges=linspace(0, max(ThompsonPrec), 20);
    binCounts=histcounts_custom(ThompsonPrec, binEdges);
    histogram_custom('BinEdges', binEdges, 'BinCounts', binCounts, graph);
    
    xlabel(graph, 'precision [nm]');
    ylabel(graph, 'N of loc')
    title(graph, 'Thompson precision, all lines')

% Wait for d to close before running to completion
uiwait(d);

%     function choice=popup_callback(popup,event)
%         idx = popup.Value;
%         popup_items = popup.String;
%         % This code uses dot notation to get properties.
%         % Dot notation runs in R2014b and later.
%         % For R2014a and earlier:
%         % idx = get(popup,'Value');
%         % popup_items = get(popup,'String');
%         choice = char(popup_items(idx,:));
%     end



    % reading the input values and returning the output
    function fitButton_callback(hObject, eventdata)
        modelFuncTypeList=get(modelFuncType_field, 'String');
        modelFuncTypeIdx=get(modelFuncType_field, 'Value');
        output.modelFuncType=modelFuncTypeList{modelFuncTypeIdx};
        output.linkerLength=str2num(get(linker_length_field, 'String'));
        output.precBounds=str2num(get(prec_bounds_field, 'String'));
        output.singlePrecValue=str2num(get(single_prec_field, 'String'));
        output.multiPrecBool=get(multi_prec_radiobutton, 'Value');
        
        delete(gcf)
    end

    % according to the radiobutton changes, this function enables or
    % disables an input filed
    function bselection(hObject, eventdata)
        if strcmp(eventdata.NewValue.Tag,'single_prec')
            set(single_prec_field, 'Enable', 'On')
        elseif strcmp(eventdata.NewValue.Tag,'multi_prec')
            set(single_prec_field, 'Enable', 'Off')
        end
        
    end

end

