classdef lineDiameter_fit_visualization
% This function plots the histogram of the original data and fitted model
% functions (both the depicting the binding site density and the convolved
% one) for comparison.
    
    methods (Static)
        
        modelFunctionStruct, histogramStruct
        
        function fittedHistFigure=plotDistribution(histogramStruct, modelFunctionStruct)
            
            % passing the settings:
            histCounts=histogramStruct.histCounts;
            histEdges=histogramStruct.histEdges;
            hist_X=(histEdges(1:end-1)+histEdges(2:end))/2;
            sampleType=modelFunctionStruct.sampleType;
            modelFuncType=modelFunctionStruct.modelFuncType;
            modelFunctionSettings=modelFunctionStruct.modelFunctionSettings;
            samplingSettings=modelFunctionStruct.samplingSettings;
            convolutionSettings=modelFunctionStruct.convolutionSettings;
            linkerType=convolutionSettings.linkerType;
            
            
            % calculate the values of the fitted distribution (convolved
            % model function)
            sample_Y_convolved_fitted=calculateDensityDistribution( sampleType, modelFuncType, modelFunctionSettings, samplingSettings, convolutionSettings);
            
            % do not extend the region for the unconvolved model function
            samplingSettings.convRad_N=0;
            
            % sampling without the extended region (because of the convolution)
            binningRegion=samplingSettings.binningRegion;
            sampling_N=samplingSettings.sampling_N;
            modelFunc_sampling_X_binningRegion=modelFunctionSampling.points(binningRegion, sampling_N, samplingSettings.convRad_N);
            
            %sample_Y_fitted=calculateModelFunction( sampleType, modelFuncType, modelFunctionSettings, samplingSettings);
            convolutionSettings.convFunc=1;
            samplingSettings.binRefinement=1;
            samplingSettings.histBinN=numel(modelFunc_sampling_X_binningRegion);
            sample_Y_fitted=calculateDensityDistribution( sampleType, modelFuncType, modelFunctionSettings, samplingSettings, convolutionSettings);
            
            
            % plot the fitted model function and the histogram data of the measured
            % distances
            
            fittedHistFigure=figure('visible','off');
            histogram_custom('BinEdges', histEdges', 'BinCounts', histCounts');
            hold on;
            % plot the fitted model function in red:
            plot(modelFunc_sampling_X_binningRegion, sample_Y_fitted, 'LineWidth', 1.5, 'color', 'red')
            % plot the fitted convolved function (that directly fits to the histogramd
            % data) in orange like color (the one that is the second or so default in
            % Matlab):
            plot(hist_X, sample_Y_convolved_fitted, 'color', [0.9290, 0.6940, 0.1250], 'LineWidth', 1.5)
            
            % figure()
            % plot(hist_X, histCounts, hist_X, sample_Y_convolved_fitted, modelFunc_sampling_X_init, sample_Y_fitted, hist_X, histcounts_custom(randn(10000,1)*57, histEdges))
            
            title(['Hist of distances, ', modelFuncType, ' fit, ', linkerType,' linker']);
            xlabel('distance [nm]')
            ylabel('N of localizations');
        end
        
        
        function fittedHistFigure = singleLine(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, densFuncBG_fitted, FVAL)
            
            fittedHistFigure=lineDiameter_fit_visualization.plotDistribution(histogramStruct, modelFunctionStruct);
            
            % passing the settings:
            histCounts=histogramStruct.histCounts;
            histEdges=histogramStruct.histEdges;
            
            % the parameters that determine the standard deviation of the
            % convolution function:
            locPrecision=modelFunctionStruct.convolutionSettings.locPrecision;
            linkerRad=modelFunctionStruct.convolutionSettings.linkerRad;
            GaussianConvSigma=modelFunctionStruct.modelFunctionSettings.GaussianConvSigma;
            
            % variance of the convolution function:
            switch modelFunctionStruct.convolutionSettings.linkerType
                case 'sphere'
                    linker_STD=linkerRad/sqrt(3);
                    convFuncVar=mean(locPrecision)^2+linker_STD^2+GaussianConvSigma^2;
                case 'Gaussian'
                    linker_STD=linkerRad/(2*sqrt(2*log(2)));
                    convFuncVar=mean(locPrecision)^2+linker_STD^2+GaussianConvSigma^2;
                otherwise
                    error('Unknows linker type')
            end
            
            % variance of the localization-central line distances:
            histX=(histEdges(1)+histEdges(2))/2;
            locVar=var(histX.*histCounts);
            
            % calculating varince of the distances, correcting the value by
            % substracting the variance of the background, linker distribution and the
            % localization precision
            bgVar=((histEdges(end)-histEdges(1))/(2*sqrt(3)))^2;
            bgN=(length(histEdges)-1)*densFuncBG_fitted;
            locN=sum(histCounts);
            locVar_bgVar_subs=(locVar*locN-bgVar*bgN)/(locN-bgN);   % substracting the variance of the background
            lineVar=locVar_bgVar_subs-convFuncVar;
            lineSTD=sqrt(lineVar);
            
            % calculating the line diameter from the variance of corrected variance of
            % the distances
            %mean(locLineDist_filtered)
            FWHM_from_var_Gauss=2*sqrt(2*log(2))*lineSTD;
            FWHM_from_var_disk=2*sqrt(3)*lineSTD;
            %FWHM_from_var_rectangular=2*sqrt(3)*lineSTD;
            
            labelXPos=[0.58];
            labelYPos=[0.91];
            
            labelText={['FWHM fit: ', sprintf('%.4g', densFuncWidth_fitted), 'nm'],...
                ['FWHM var: ', sprintf('%.4g/%.4g', FWHM_from_var_Gauss, FWHM_from_var_disk), 'nm'],...
                ['residual: ', sprintf('%.4g', FVAL)]};
            
            text( labelXPos, labelYPos, labelText, 'Units', 'normalized')
        end
        
        
        
        function fittedHistFigure = doubleLine(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, lineDistance_fitted, FVAL)
            
            fittedHistFigure=lineDiameter_fit_visualization.plotDistribution(histogramStruct, modelFunctionStruct);
            
            labelXPos=[0.58];
            labelYPos=[0.91];
                
                labelText={['FWHM fit: ', sprintf('%.4g', densFuncWidth_fitted), 'nm'],...
                    ['line dist.: ', sprintf('%.4g', lineDistance_fitted), 'nm'],...
                    ['residual: ', sprintf('%.4g', FVAL)]};
                
                text( labelXPos, labelYPos, labelText, 'Units', 'normalized')
        end
        
        
        
        function fittedHistFigure = gap(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, GaussianConvSigma_fitted, FVAL)
            
            fittedHistFigure=lineDiameter_fit_visualization.plotDistribution(histogramStruct, modelFunctionStruct);
            
            labelXPos=[0.58];
            labelYPos=[0.91];
            
            if ~modelFunctionStruct.convolutionSettings.iterGaussianConvBool
                labelText={['gap width: ', sprintf('%.4g', densFuncWidth_fitted), 'nm'],...
                    ['residual: ', sprintf('%.4g', FVAL)]};
                
                ax = gca;
                %set(ax, 'XLim', [-Inf Inf]);
                set(ax, 'YLim', [0 1.5*max(histogramStruct.histCounts)]);
            else
                labelText={['gap width: ', sprintf('%.4g', densFuncWidth_fitted), 'nm'],...
                    ['residual: ', sprintf('%.4g', FVAL)],...
                    ['Gaussian blur: ', sprintf('%.4g', GaussianConvSigma_fitted), 'nm']};
                
                ax = gca;
                %set(ax, XLim, [-Inf Inf]);
                set(ax, 'YLim', [0 1.5*max(histogramStruct.histCounts)]);
            end
            
            text( labelXPos, labelYPos, labelText, 'Units', 'normalized')
        end
        
    end
end

