classdef modelFunctionSampling
    % These methods determine how the model function should be
    % sampled or perform the sampling.
    
    methods(Static)
        function [histBinN, binningRegion, sampling_N] = settings(histEdges ,binRefinement)
            % This function determines the how the model function should be
            % sampled based on the histogram bin coordinates of the
            % original data on the bin refinement. It only determines the
            % region that should be binned later (larger region should be
            % sampled because of the convolution).
            
            % number of bins in the original data:
            histBinN=numel(histEdges)-1;
            
            % number of bins after the refinement
            sampling_N=histBinN*binRefinement;
            
            % size of this region:
            binningRegion=[histEdges(1), histEdges(end)];
            
        end
        
        function sampling_dX=dX(binningRegion, sampling_N)
            % This function calculates the refined bins sizes belonging to
            % the smpling points.
            
            sampling_dX=(binningRegion(2)-binningRegion(1))/sampling_N;
        end
        
        function [sampling_X, sampling_dX, sampling_extended_N]=points(binningRegion, sampling_N, convRad_N)
            % This function returnes the centers, the numbers and the sizes of the sampling bins of the model function.
            % They are determined from the histogram of the original data
            % and from the radius of the convolution (in  number of sampling points) function that the
            % model function shall be convolved.
            
            
            % get the centers and the sizes of the sampling bins:
            [samplingBoundaries_left, samplingBoundaries_right, sampling_dX]=modelFunctionSampling.boundaries(binningRegion, sampling_N, convRad_N);
            
            % the sampling coordinates (centers of the refined bins):
            sampling_X=(samplingBoundaries_left+samplingBoundaries_right)/2;
            
            % number of the sampling points:
            sampling_extended_N=numel(sampling_X);
        end
        
        
        function [samplingBoundaries_left, samplingBoundaries_right, sampling_dX]=boundaries(binningRegion, sampling_N, convRad_N)
            % This function returnes the left and the right boundaries of the sampling bins (the
            % regions belonging to each sampling points) of the model function.
            % They are determined from the histogram of the original data
            % and from the radius of the convolution function (in  number of sampling points) that the
            % model function shall be convolved.
            
            % prepare the sampling:
            sampling_dX=modelFunctionSampling.dX(binningRegion, sampling_N);
            samplingBoundaries_N=sampling_N+2*convRad_N+1;
            samplingRegion=[binningRegion(1)-convRad_N*sampling_dX, binningRegion(2)+convRad_N*sampling_dX];
            samplingBoundaries=linspace(samplingRegion(1), samplingRegion(2), samplingBoundaries_N)';
            
            % calculate the bins' boundaries:
            samplingBoundaries_left=samplingBoundaries(1:end-1);
            samplingBoundaries_right=samplingBoundaries(2:end);
        end
    end
end

