classdef modelFunctions
% These methods hold the definitions of the models functions and some other
% auxilary functions to required for their sampling, creation and
% manipulation.
% These methods do not perform any model function height or area
% adjustment, they return only the normalized functions. It is the user's
% task to set the desired height or area. Because of it, it does not add
% any background, it is also the user's task.
    
% These methods first calculate the function values of the supposed
% model function (normalized). In case of 'Gaussian' and
% 'Lorentzian' binding site density functions, the output values are
% calculated by sampling these functions at the given sampling
% coordinates while in case of 'circle', 'disk', 'ring' and 'rectangle'
% functions, the output values are calculated by integrating the functions
% around the the given sampling coordinates. In case of the 'Gaussian',
% 'Lorentzian' and the 'Disk' function, the model funtion width parameter
% that has to be given is the full width at half maximum, while in case of
% 'rectangle', 'circle', 'disk' and 'ring' model functions, it is the full width.

    methods(Static)
        
        function modelFunc_Y=circle(modelFunctionSettings, samplingSettings)
            % This model function describes the projection of a cylindrical
            % surface.
            % Samples the model function by integrating it over the
            % sampling bins.
            
            % parameters of the model function:
            radius=modelFunctionSettings.width/2;
            position=modelFunctionSettings.position;
            
            % get the sampling points:
            [modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, ~]=modelFunctionSampling.boundaries(samplingSettings.binningRegion, samplingSettings.sampling_N, samplingSettings.convRad_N);
            
            % check which boundary ponts are affected by the model function
            % and calculate the polar angle of the boundary points:
            [theta_left, theta_right ,sampXWithinBool, boundIdx_left, boundIdx_right]=...
                modelFunctions.circlePreparation(radius, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, position);
            
            % initialize the model function:
            modelFunc_Y=zeros(size(modelFunc_sampBoundaries_left));
            
            % model function sampling values where the sampling region is fully within the
            % rectangle:
            modelFunc_Y(sampXWithinBool)=radius*2*(theta_left(sampXWithinBool)-theta_right(sampXWithinBool));
            
            % model function sampling values where the sampling region is only partially within
            % the rectangle:
            if boundIdx_left~=boundIdx_right
                modelFunc_Y(boundIdx_left)=radius*2*abs(pi-theta_right(boundIdx_left));
                modelFunc_Y(boundIdx_right)=radius*2*abs(theta_left(boundIdx_right)-0);
            else
                modelFunc_Y(boundIdx_right)=radius*2*pi;
            end
            
            % normalize the model function:
            modelFunc_Y=modelFunc_Y/(2*radius*pi);
            
        end
        
        
        function modelFunc_Y=disk(modelFunctionSettings, samplingSettings)
            % This function describes the projection of a disk-like
            % structure (solid cylinder).
            % Samples the model function by integrating it over the
            % sampling bins.
            
            
            % parameters of the model function:
            radius=modelFunctionSettings.width/2;
            position=modelFunctionSettings.position;
            
            % get the sampling points:
            [modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, ~]=modelFunctionSampling.boundaries(samplingSettings.binningRegion, samplingSettings.sampling_N, samplingSettings.convRad_N);
            
            % check which boundary ponts are affected by the model function
            % and calculate the polar angle of the boundary points:
            [theta_left, theta_right,sampXWithinBool, boundIdx_left, boundIdx_right]=...
                modelFunctions.circlePreparation(radius, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, position);
            
            % initialize the model function:
            modelFunc_Y=zeros(size(modelFunc_sampBoundaries_left));
            
            % model function sampling values where the sampling region is fully within the
            % rectangle:
            modelFunc_Y=zeros(size(modelFunc_Y));
            modelFunc_Y(sampXWithinBool)=radius^2*((theta_left(sampXWithinBool)-sin(2*theta_left(sampXWithinBool))/2)-(theta_right(sampXWithinBool)-sin(2*theta_right(sampXWithinBool))/2));
            
            % model function sampling values where the sampling region is only partially within
            % the rectangle:
            if boundIdx_left~=boundIdx_right
                modelFunc_Y(boundIdx_left)=radius^2*(pi-(theta_right(boundIdx_left)-sin(2*theta_right(boundIdx_left))/2));
                modelFunc_Y(boundIdx_right)=radius^2*(theta_left(boundIdx_right)-sin(2*theta_left(boundIdx_right))/2);
            else
                modelFunc_Y(boundIdx_right)=radius^2*pi;
            end
            
            % normalize the model function:
            modelFunc_Y=modelFunc_Y/(radius^2*pi);
            
        end
        
        
        function  modelFunc_Y=ring(modelFunctionSettings, samplingSettings)
            % This function models the projection of a ring-like (hollow
            % cylinder) structure. It is modelled as the difference of the
            % coaxial cylinders.
            % Samples the model function by integrating it over the
            % sampling bins.
            
            % parameters of the model function:
            % outer disk function:
            modelFunctionSettings_outer.width=modelFunctionSettings.width+modelFunctionSettings.ringThickness/2;
            modelFunctionSettings_outer.position=modelFunctionSettings.position;
            % inner disk:
            modelFunctionSettings_inner.width=modelFunctionSettings.width-modelFunctionSettings.ringThickness/2;
            modelFunctionSettings_inner.position=modelFunctionSettings.position;
            
            % the larger cylinders's projection:
            modelFunc_Y_outer=modelFunctions.disk(modelFunctionSettings_outer, samplingSettings);
            
            % calculate the area proportional to the radius:
            area_outer=(modelFunctionSettings_outer.width/2)^2*pi;
            modelFunc_Y_outer=modelFunc_Y_outer*area_outer;
            
            % the inner cylinder's projection:
            modelFunc_Y_inner=modelFunctions.disk(modelFunctionSettings_inner, samplingSettings);
            
            % calculate the area proportional to the radius:
            area_inner=(modelFunctionSettings_inner.width/2)^2*pi;
            modelFunc_Y_inner=modelFunc_Y_inner*area_inner;
            
            % the differece of the two cylinder's projection:
            modelFunc_Y=modelFunc_Y_outer-modelFunc_Y_inner;
            
            % normalize the model function:
            modelFunc_Y=modelFunc_Y/(area_outer-area_inner);
            
        end
        
        
        function modelFunc_Y=Gaussian(modelFunctionSettings, samplingSettings)
            % Sampling the Gaussian function by taking its values at the
            % sampling points. Good approximation for the Gaussian. Integrating
            % it over the sampling regions would be overcomplication.
            
            % parameters of the model function:
            position=modelFunctionSettings.position;
            GaussSTD=modelFunctionSettings.width/2.3548;
            
            % get the sampling points:
            [modelFunc_sampling_X, ~, ~]=modelFunctionSampling.points(samplingSettings.binningRegion, samplingSettings.sampling_N, samplingSettings.convRad_N);
            
            % dye density following Gaussian distribution (normalized):
            modelFunc_Y=1/(GaussSTD*sqrt(2*pi))*exp(-(modelFunc_sampling_X-position).^2/(2*GaussSTD^2));
            
        end
        
        
        function modelFunc_Y=Lorentzian(modelFunctionSettings, samplingSettings)
            % Sampling the Lorentzian function by taking its values at the
            % sampling points. No idea how good this approxiamtion is, but
            % should be sufficient if it works well for the Gaussian.
            
            % parameters of the model function:
            position=modelFunctionSettings.position;
            gamma=modelFunctionSettings.width/2;
            
            % get the sampling points:
            [modelFunc_sampling_X, ~, ~]=modelFunctionSampling.points(samplingSettings.binningRegion, samplingSettings.sampling_N, samplingSettings.convRad_N);
            
            % dye density following Cauchy distrinution (a Lorentzian
            % function):
            modelFunc_Y=1./(pi*gamma*(1+((modelFunc_sampling_X-position)/gamma).^2));
            
        end
        
        
        function modelFunc_Y=rectangle(modelFunctionSettings, samplingSettings)
            % This function describes the projection of a rectangular
            % structure (top-hat).
            % Samples the model function by integrating it over the
            % sampling bins.
            
            % pass the model function settings:
            width=modelFunctionSettings.width;
            height=1;
            position=modelFunctionSettings.position;
            
            % get the sampling points:
            [modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, ~]=modelFunctionSampling.boundaries(samplingSettings.binningRegion, samplingSettings.sampling_N, samplingSettings.convRad_N);
            
            % initialize the model function:
            modelFunc_Y=zeros(size(modelFunc_sampBoundaries_left));
            
            
            % left and right bounds of the rectangle:
            bound_left=-width/2+position;
            bound_right=width/2+position;
            
            % sampling values where the sampling bin is fully within the
            % rectangle:
            modelFunc_Y(modelFunc_sampBoundaries_left>=bound_left & modelFunc_sampBoundaries_right<=bound_right)=height;
            
            % left and right sampling regions indices where the sampling
            % bins are only partially within the rectangle:
            R_boundIdx_left=find(modelFunc_sampBoundaries_left<bound_left & modelFunc_sampBoundaries_right>bound_left);
            R_boundIdx_right=find(modelFunc_sampBoundaries_left<bound_right & modelFunc_sampBoundaries_right>bound_right);
            
            % sampling values where the sampling bin is only partially within
            % the rectangle:
            if R_boundIdx_left~=R_boundIdx_right
                modelFunc_Y(R_boundIdx_left)=...
                    (bound_left-modelFunc_sampBoundaries_right(R_boundIdx_left))/...
                    (modelFunc_sampBoundaries_left(R_boundIdx_left)-modelFunc_sampBoundaries_right(R_boundIdx_left))*height;
                modelFunc_Y(R_boundIdx_right)=...
                    (modelFunc_sampBoundaries_left(R_boundIdx_right)-bound_right)/...
                    (modelFunc_sampBoundaries_left(R_boundIdx_right)-modelFunc_sampBoundaries_right(R_boundIdx_right))*height;
            else
                modelFunc_Y(R_boundIdx_right)=...
                    (bound_left-bound_right)/...
                    (modelFunc_sampBoundaries_left(R_boundIdx_right)-modelFunc_sampBoundaries_right(R_boundIdx_right))*height;
            end
            
            % normalize the model function:
            modelFunc_Y=modelFunc_Y/(width*height);
            
            
        end
        
        
        function modelFunc_Y=rectangularGap(modelFunctionSettings, samplingSettings)
            % This function describes the projection of a rimmed gap
            % structure.
            
            % pass the model function settings:
            gapWidth=modelFunctionSettings.gapWidth;
            gapDepth=modelFunctionSettings.gapDepth;
            position=modelFunctionSettings.position;
            rimLeftWidth=modelFunctionSettings.rimWidth;
            rimLeftHeight=modelFunctionSettings.rimHeight;
            rimLeftPos=position-gapWidth/2-rimLeftWidth/2;
            rimRightWidth=modelFunctionSettings.rimWidth;
            rimRightHeight=modelFunctionSettings.rimHeight;
            rimRightPos=position+gapWidth/2+rimRightWidth/2;
            area=1;
            
            % get the sampling points
            modelFunc_sampling_dX=modelFunctionSampling.dX(samplingSettings.binningRegion, samplingSettings.sampling_N);
            
            % get the gap:
            modelFunctionSettings_gap.width=gapWidth;
            modelFunctionSettings_gap.position=position;
            modelFunctionSettings_gap.background=0;
            modelFunc_Y_gap=gapDepth*(1-modelFunctions.rectangle(modelFunctionSettings_gap, samplingSettings));
            
            % get the left rim:
            modelFunctionSettings_rimLeft.width=rimLeftWidth;
            modelFunctionSettings_rimLeft.position=rimLeftPos;
            modelFunctionSettings_rimLeft.background=0;
            modelFunc_Y_rimLeft=rimLeftHeight*modelFunctions.rectangle(modelFunctionSettings_rimLeft, samplingSettings);
            
            % get the left rim:
            modelFunctionSettings_rimRight.width=rimRightWidth;
            modelFunctionSettings_rimRight.position=rimRightPos;
            modelFunctionSettings_rimRight.background=0;
            modelFunc_Y_rimRight=rimRightHeight*modelFunctions.rectangle(modelFunctionSettings_rimRight, samplingSettings);
            
            % the model function:
            modelFunc_Y=modelFunc_Y_gap+modelFunc_Y_rimLeft+modelFunc_Y_rimRight;
            
            % set the area under it numerically:
            modelFunc_Y=modelFunctions.setArea(modelFunc_Y, modelFunc_sampling_dX, area, modelFunctionSettings.background);
            
        end
       
        
        
        function modelFunc_Y=setArea(modelFunc_Y, modelFunc_sampling_dX, area, background)
            % Changes the area under the model function to the desired
            % value.
            % The area corresponds to the area under sampled model function plus the background.
            % It does not add the background to the model function, only
            % scales it.
            
            % area iunder the background:
            if numel(background)==1
                area_background=background*numel(modelFunc_Y)*modelFunc_sampling_dX;
            elseif numel(background)==numel(modelFunc_Y)
                area_background=sum(background)*modelFunc_sampling_dX;
            else
                error('Invalid number of data for the background')
            end
            
            % area under the model function:
            area_modelFunc=sum(modelFunc_Y)*modelFunc_sampling_dX;
            
            % the scaling factor for the model function:
            scaling=(area-area_background)/area_modelFunc;
            
            % the changed model function:
            modelFunc_Y=scaling*modelFunc_Y;
        end
        
        function modelFunc_Y=setAHeight(modelFunc_Y, height)
            % Changes the height of the model function to the desired
            % value.
            % The height corresponds to the sampled model function maximum value
            % without background.

            
            % the changed model function:
            modelFunc_Y=height/max(modelFunc_Y)*modelFunc_Y;
        end
        
    end
    
    
    methods(Static, Access=private)
        
        function [theta_left, theta_right,sampXWithinBool, boundIdx_left, boundIdx_right]=circlePreparation(circleRad, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, position)
            % This function examines which sampling points are affected by
            % the disk of circle pattern.
            
            % angle value for the boundary points outside the circle, should be
            % the same for the "left" and "right" boundary points:
            thetaOutsideValue=0;       % let it be
            
            % finding which left boundaries are winthin the circle:
            sampXWithinBool_left=false(size(modelFunc_sampBoundaries_left));
            sampXWithinBool_left(modelFunc_sampBoundaries_left>-circleRad+position & modelFunc_sampBoundaries_left<circleRad+position)=true;
            
            % polar angle for the left boundray points:
            theta_left(~sampXWithinBool_left,1)=thetaOutsideValue;
            theta_left(sampXWithinBool_left,1)=acos((modelFunc_sampBoundaries_left(sampXWithinBool_left)-position)/circleRad);
            
            % finding which right boundaries are within the circle:
            sampXWithinBool_right=false(size(modelFunc_sampBoundaries_right));
            sampXWithinBool_right(modelFunc_sampBoundaries_right>-circleRad+position & modelFunc_sampBoundaries_right<circleRad+position)=true;
            
            % polar angle for the right boundray points:
            theta_right(~sampXWithinBool_right,1)=thetaOutsideValue;
            theta_right(sampXWithinBool_right,1)=acos((modelFunc_sampBoundaries_right(sampXWithinBool_right)-position)/circleRad);
            
            % boolean vector for the sampling points within the circle:
            sampXWithinBool=sampXWithinBool_left & sampXWithinBool_right;
            
            % indices of the left and right sampling points of the circle where
            % the sampling region is only partly within the circle:
            boundIdx_left=find(~sampXWithinBool_left & sampXWithinBool_right);
            boundIdx_right=find(sampXWithinBool_left & ~sampXWithinBool_right);
        end
        
    end
end

