function [fittedCoeffs_linear, exitflag] = linearFitting_fsolve(initLine_X_coords, initLine_Y_coords, initCoeffs_linear)
% This function fits a linear on the scattered points by minimizing the sum of
% square distances and not with the simple but unreliable (when the line is
% close to vertical) method of fitting based on the sum of square of "y" 
% differences which is taught in universities and is implemented in Matlab
% 
% function [fittedCoeffs_linear, exitflag] = linearFitting_fsolve(initLine_X_coords, initLine_Y_coords, initCoeffs_linear)
% 
% Inputs:
% 
% - initLine_X_coords: the "x" coordinates of the point the initial line shall be fitted on
% - initLine_Y_coords: the "y" coordinates of the point the initial line shall be fitted on
% - initCoeffs_linear: 1x2 array containing the initial parameters parameters for the fitting (slope - 1st element ; intercept -2nd element)
% 
% Outputs:
% 
% - fittedCoeffs_linear: 1x2 array containing the slope (1st element) and intercept (2nd element) of the fitted line 
% - exitflag: returns whether the iteration was succesful (1) or unszccesful (-1)



% FOR TESTING
% initLine_X_coords=10*rand(1000,1);
% initLine_Y_coords=0.3*initLine_X_coords+4+0.3*randn(1000,1);
% initParams_linear=[-0.4,3.9];

N=numel(initLine_X_coords);         % number of points
sum_X_i=sum(initLine_X_coords);     % sum of "x" coordinates of points
sum_Y_i=sum(initLine_Y_coords);     % sum of "y" coordinates of points
sum_X_i_2=sum(initLine_X_coords.^2);    % sum of squares "x" coordinates of points
sum_Y_i_2=sum(initLine_Y_coords.^2);    % sum of squares "y" coordinates of points
sum_X_i_Y_i=sum(initLine_X_coords.*initLine_Y_coords);  % sum of product of "x" and "y" coordinates of points

    % the sum of squares of point distances from the linear line
    function F=sumSquareDistances(x)
        % substacting the inscision height from the "y" coordinates theN
        % rotating the points with the line angle clockwise and then their
        % distance from the line equals their distances from the "x" axis
        % (which are their modified "y" coordinates)
        % x(1): angle of the linear line
        % x(2): height is incision of the linear line with the "y" axis
        F=sin(x(1))^2*sum_X_i_2+cos(x(1))^2*((sum_Y_i_2-2*sum_Y_i*x(2)+N*x(2)^2))-sin(2*x(1))*(sum_X_i_Y_i-sum_X_i*x(2));
    end

    % checking the local minima with the second partial derivates
    function F=second_derivate_test(x)
        % https://en.wikipedia.org/wiki/Second_partial_derivative_test
        % x(1): angle of the linear line
        % x(2): height is incision of the linear line with the "y" axis
        f_xx=2*cos(2*x(1))*(sum_X_i_2-(sum_Y_i_2-2*sum_Y_i*x(2)+N*x(2)^2))+4*sin(2*x(1))*(sum_X_i_Y_i-sum_X_i*x(2));
        f_yy=2*cos(x(1))^2*(N);
        f_xy=2*sin(2*x(1))*(sum_Y_i-N*x(2))+2*cos(2*x(1))*sum_X_i;
        D=f_xx*f_yy-f_xy^2;
        
        F= D>0 && f_xx>0;
    end

% calculating the line angle from the slope
initParams_linear(1,1)=atan(initCoeffs_linear(1,1));
%initParams_linear(1,2)=initCoeffs_linear(1,2);
initParams_linear(1,2)=min(initLine_Y_coords);

% probing several initial line angles and inscision heights
% line angle steps
delta_angle=5*pi/180;
N_angle_steps=ceil(pi/delta_angle);
% inscision height steps
N_h_steps=10;
delta_h=(max(initLine_Y_coords)-min(initLine_Y_coords))/(N_h_steps-1);
%initializing
sumSquareDistValues=zeros(N_angle_steps, N_h_steps, 1);
localMinimaBoolVect=false(N_angle_steps, N_h_steps, 1);
fittedParamsVect_linear=zeros(N_angle_steps, N_h_steps, 2);

try
    % in case of Matlab
    options=optimoptions('fsolve','Display', 'none');
catch
    % in case of Octave
    options=optimset('Display', 'none');
    
    % Octave does not have 'optimoptions'. Bost should set the default
    % iteration parameres, but I am not exactly sure, the documentation was
    % not clear enough in this matter. Both works, but I don't have the
    % time to test whether they give the exactly same results...
end

% the probing
for idxH=1:N_h_steps
    for idxAngle=1:N_angle_steps
        % changing the initial line angle
        actInitParams_linear=initParams_linear+(idxAngle-1)*[delta_angle, 0]+(idxH-1)*[0, delta_h];
        
        % finding a local mimimum/maximum
        fittedParamsVect_linear(idxAngle, idxH, :)=fsolve(@(x) linearFit(x, N, sum_X_i, sum_Y_i, sum_X_i_2, sum_Y_i_2, sum_X_i_Y_i),actInitParams_linear, options);
        
        % function value and second order derivate at the found local extrema
        sumSquareDistValues(idxAngle, idxH)=sumSquareDistances(fittedParamsVect_linear(idxAngle, idxH, :));
        localMinimaBoolVect(idxAngle, idxH)=second_derivate_test(fittedParamsVect_linear(idxAngle, idxH, :));
        
        % FOR TESTING
        %     x_fittedLine=(-100:1:100)*cos(fittedParamsVect_linear(idxAngle, 1));
        %     y_fittedLine=(-100:1:100)*sin(fittedParamsVect_linear(idxAngle, 1))+fittedParamsVect_linear(idxAngle, 2);
        %     figure()
        %     plot(x_fittedLine, y_fittedLine);
        %     hold on;
        %     scatter(initLine_X_coords, initLine_Y_coords);
        %     hold off;
    end
end
% choosing the best line angle
[~, minIdx_x_temp]=min(sumSquareDistValues);
[~, minIdx_y]=min(min(sumSquareDistValues));
minIdx_x=minIdx_x_temp(minIdx_y);

% sumSquareDistValues(minIdx)
% sumSquareDistances(initParams_linear)

% FOR TESTING
% x_fittedLine=(-100:1:100)*cos(fittedParamsVect_linear(minIdx_x, minIdx_y, 1));
% y_fittedLine=(-100:1:100)*sin(fittedParamsVect_linear(minIdx_x, minIdx_y, 1))+fittedParamsVect_linear(minIdx_x, minIdx_y, 2);
% figure()
% plot(x_fittedLine, y_fittedLine);
% hold on;
% scatter(initLine_X_coords, initLine_Y_coords);
% hold off;

% checking if everything is ok, whether the fitted params are local minima
% and whether they are better than the initial guess
if localMinimaBoolVect(minIdx_x, minIdx_y) && sumSquareDistValues(minIdx_x, minIdx_y)<sumSquareDistances(initParams_linear)
    fittedParams_linear(1,:)=fittedParamsVect_linear(minIdx_x, minIdx_y, :);
    
    % calculating the slope from the line angle
    fittedCoeffs_linear(1,1)=tan(fittedParams_linear(1,1));
    fittedCoeffs_linear(1,2)=fittedParams_linear(1,2);
    exitflag=1;
else
    % if it could not fit the line, return this
    disp('Something sinister is going on here. Could not fit the linear.')
    fittedCoeffs_linear=[];
    exitflag=-1;
end



end

% partial derivates of  the sum of squares of point distances from the
% linear line with respect to line angle and the incision height
% this function needs to equal zero to achieve fitting
function F=linearFit(x, N, sum_X_i, sum_Y_i, sum_X_i_2, sum_Y_i_2, sum_X_i_Y_i)
% x(1): angle of the linear line
% x(2): height is incision of the linear line with the "y" axis
F(1,1)=-2*cos(x(1))^2*(sum_Y_i-N*x(2))+sin(2*x(1))*sum_X_i;
F(1,2)=sin(2*x(1))*(sum_X_i_2-(sum_Y_i_2-2*sum_Y_i*x(2)+N*x(2)^2))-2*cos(2*x(1))*(sum_X_i_Y_i-sum_X_i*x(2));
end